@@ -3,7 +3,7 @@ import logging
 from datetime import datetime, timezone, timedelta
 
 from sqlalchemy import and_
-from sqlalchemy.exc import IntegrityError
+from sqlalchemy.exc import IntegrityError, InvalidRequestError
 from sqlalchemy.orm.exc import ObjectDeletedError
 
 from vwsfriend.model.trip import Trip
@@ -25,7 +25,7 @@ class TripAgent():
     def __init__(self, session, vehicle, updateInterval, privacy):  # noqa: C901
         self.mode = TripAgent.Mode.NONE
         self.session = session
-        self.vehicle = vehicle
+        self.vehicle = session.merge(vehicle)
         self.privacy = privacy
         if Privacy.NO_LOCATIONS in self.privacy:
             LOG.info(f'Privacy option \'no-locations\' is set. Vehicle {self.vehicle.vin} will not record start and destination location')
@@ -42,7 +42,6 @@ class TripAgent():
                 if Privacy.NO_LOCATIONS not in self.privacy:
                     self.lastParkingPositionLatitude = self.trip.destination_position_latitude
                     self.lastParkingPositionLongitude = self.trip.destination_position_longitude
-                self.session.commit()
             else:
                 LOG.info(f'Vehicle {self.vehicle.vin} has still an open trip during startup, closing it now')
             self.trip = None
@@ -76,7 +75,8 @@ class TripAgent():
                         self.vehicle.weConnectVehicle.domains['readiness']['readinessStatus'].connectionState.isActive \
                             .addObserver(self.__onIsActiveChanged, AddressableLeaf.ObserverEvent.VALUE_CHANGED, onUpdateComplete=True)
                         self.vehicle.weConnectVehicle.domains['readiness']['readinessStatus'].connectionState.isActive \
-                            .addObserver(self.__onIsActiveDisabled, AddressableLeaf.ObserverEvent.DISABLED, onUpdateComplete=True)
+                            .addObserver(self.__onIsActiveEnabledDisabled, (AddressableLeaf.ObserverEvent.ENABLED | AddressableLeaf.ObserverEvent.DISABLED),
+                                         onUpdateComplete=True)
                         LOG.info(f'Vehicle {self.vehicle.vin} provides isActive flag in readinessStatus and thus allows to record trips with several minutes'
                                  ' inaccuracy')
                         self.mode = TripAgent.Mode.READINESS_STATUS
@@ -120,6 +120,9 @@ class TripAgent():
             except ObjectDeletedError:
                 LOG.warning('Last trip entry was deleted')
                 self.trip = None
+            except InvalidRequestError:
+                LOG.warning('Last trip entry was not persisted')
+                self.trip = None
 
         if self.mode == TripAgent.Mode.PARKING_POSITION:
             if element.parent.error.enabled:
@@ -145,12 +148,12 @@ class TripAgent():
                 odometerMeasurement = self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus']
                 if odometerMeasurement.odometer.enabled and odometerMeasurement.odometer is not None:
                     self.trip.start_mileage_km = odometerMeasurement.odometer.value
-            try:
-                with self.session.begin_nested():
+            with self.session.begin_nested():
+                try:
                     self.session.add(self.trip)
-                self.session.commit()
-            except IntegrityError as err:
-                LOG.warning('Could not add trip to the database, this is usually due to an error in the WeConnect API (%s)', err)
+                except IntegrityError as err:
+                    LOG.warning('Could not add trip to the database, this is usually due to an error in the WeConnect API (%s)', err)
+            self.session.commit()
             LOG.info(f'Vehicle {self.vehicle.vin} started a trip')
 
     def __onCarCapturedTimestampChanged(self, element, flags):
@@ -181,32 +184,31 @@ class TripAgent():
                 self.lastParkingPositionLongitude = parkingPosition.longitude.value
             if self.trip is not None:
                 if parkingPosition.carCapturedTimestamp.enabled and parkingPosition.carCapturedTimestamp.value is not None:
-                    if parkingPosition.carCapturedTimestamp.value > self.trip.startDate:
-                        self.trip.endDate = parkingPosition.carCapturedTimestamp.value
-                        if Privacy.NO_LOCATIONS not in self.privacy:
-                            if parkingPosition.latitude.enabled and parkingPosition.latitude.value is not None \
-                                    and parkingPosition.longitude.enabled and parkingPosition.longitude.value is not None:
-                                self.trip.destination_position_latitude = parkingPosition.latitude.value
-                                self.trip.destination_position_longitude = parkingPosition.longitude.value
-                                self.trip.destination_location = locationFromLatLonWithGeofence(self.session, parkingPosition.latitude.value,
-                                                                                                parkingPosition.longitude.value)
+                    with self.session.begin_nested():
+                        if parkingPosition.carCapturedTimestamp.value > self.trip.startDate:
+                            self.trip.endDate = parkingPosition.carCapturedTimestamp.value
+                            if Privacy.NO_LOCATIONS not in self.privacy:
+                                if parkingPosition.latitude.enabled and parkingPosition.latitude.value is not None \
+                                        and parkingPosition.longitude.enabled and parkingPosition.longitude.value is not None:
+                                    self.trip.destination_position_latitude = parkingPosition.latitude.value
+                                    self.trip.destination_position_longitude = parkingPosition.longitude.value
+                                    self.trip.destination_location = locationFromLatLonWithGeofence(self.session, parkingPosition.latitude.value,
+                                                                                                    parkingPosition.longitude.value)
 
-                        if self.vehicle.weConnectVehicle.statusExists('measurements', 'odometerStatus') \
-                                and self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus'].enabled:
-                            odometerMeasurement = self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus']
-                            if odometerMeasurement.odometer.enabled and odometerMeasurement.odometer is not None:
-                                self.trip.end_mileage_km = odometerMeasurement.odometer.value
+                            if self.vehicle.weConnectVehicle.statusExists('measurements', 'odometerStatus') \
+                                    and self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus'].enabled:
+                                odometerMeasurement = self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus']
+                                if odometerMeasurement.odometer.enabled and odometerMeasurement.odometer is not None:
+                                    self.trip.end_mileage_km = odometerMeasurement.odometer.value
 
-                        self.session.commit()
-                        self.trip = None
+                            self.trip = None
 
-                        LOG.info(f'Vehicle {self.vehicle.vin} ended a trip')
-                    else:
-                        with self.session.begin_nested():
+                            LOG.info(f'Vehicle {self.vehicle.vin} ended a trip')
+                        else:
                             self.session.delete(self.trip)
-                        self.session.commit()
-                        self.trip = None
-                        LOG.info(f'Previously started trip for {self.vehicle.vin} was invalid. Deleting it now.')
+                            self.trip = None
+                            LOG.info(f'Previously started trip for {self.vehicle.vin} was invalid. Deleting it now.')
+                    self.session.commit()
             else:
                 if flags is not None:
                     LOG.info(f'Vehicle {self.vehicle.vin} provides a parking position, but no trip was started (this is ok during startup)')
@@ -218,6 +220,8 @@ class TripAgent():
             except ObjectDeletedError:
                 LOG.warning('Last trip entry was deleted')
                 self.trip = None
+            except InvalidRequestError:
+                LOG.warning('Last trip entry was not persisted')
 
         if self.mode == TripAgent.Mode.READINESS_STATUS:
             if self.vehicle.weConnectVehicle.statusExists('charging', 'plugStatus'):
@@ -234,23 +238,24 @@ class TripAgent():
                         odometerMeasurement = self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus']
                         if odometerMeasurement.odometer.enabled and odometerMeasurement.odometer is not None:
                             self.trip.start_mileage_km = odometerMeasurement.odometer.value
-                    try:
-                        with self.session.begin_nested():
+                    with self.session.begin_nested():
+                        try:
                             self.session.add(self.trip)
-                        self.session.commit()
-                    except IntegrityError as err:
-                        LOG.warning('Could not add trip to the database, this is usually due to an error in the WeConnect API (%s)', err)
+                        except IntegrityError as err:
+                            LOG.warning('Could not add trip to the database, this is usually due to an error in the WeConnect API (%s)', err)
+                    self.session.commit()
                     LOG.info(f'Vehicle {self.vehicle.vin} started a trip')
             else:
                 if self.trip is not None:
-                    self.trip.endDate = datetime.utcnow().replace(tzinfo=timezone.utc, microsecond=0)
-                    if self.vehicle.weConnectVehicle.statusExists('measurements', 'odometerStatus') \
-                            and self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus'].enabled:
-                        odometerMeasurement = self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus']
-                        if odometerMeasurement.odometer.enabled and odometerMeasurement.odometer is not None:
-                            self.trip.end_mileage_km = odometerMeasurement.odometer.value
-
+                    with self.session.begin_nested():
+                        self.trip.endDate = datetime.utcnow().replace(tzinfo=timezone.utc, microsecond=0)
+                        if self.vehicle.weConnectVehicle.statusExists('measurements', 'odometerStatus') \
+                                and self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus'].enabled:
+                            odometerMeasurement = self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus']
+                            if odometerMeasurement.odometer.enabled and odometerMeasurement.odometer is not None:
+                                self.trip.end_mileage_km = odometerMeasurement.odometer.value
                     self.session.commit()
+
                     self.trip = None
 
                     LOG.info(f'Vehicle {self.vehicle.vin} ended a trip')
@@ -258,13 +263,15 @@ class TripAgent():
                     if flags is not None:
                         LOG.info(f'Vehicle {self.vehicle.vin} reports to be inactive, but no trip was started (this is ok during startup)')
 
-    def __onIsActiveDisabled(self, element, flags):  # noqa: C901
+    def __onIsActiveEnabledDisabled(self, element, flags):  # noqa: C901
         if self.trip is not None:
             try:
                 self.session.refresh(self.trip)
             except ObjectDeletedError:
                 LOG.warning('Last trip entry was deleted')
                 self.trip = None
+            except InvalidRequestError:
+                LOG.warning('Last trip entry was not persisted')
 
         if self.mode == TripAgent.Mode.READINESS_STATUS:
             if self.vehicle.weConnectVehicle.statusExists('charging', 'plugStatus'):
@@ -272,21 +279,39 @@ class TripAgent():
                 if plugStatus.enabled and plugStatus.plugConnectionState.enabled \
                         and plugStatus.plugConnectionState.value == PlugStatus.PlugConnectionState.CONNECTED:
                     return
-            if self.trip is not None:
-                self.trip.endDate = datetime.utcnow().replace(tzinfo=timezone.utc, microsecond=0)
-                if self.vehicle.weConnectVehicle.statusExists('measurements', 'odometerStatus') \
-                        and self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus'].enabled:
-                    odometerMeasurement = self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus']
-                    if odometerMeasurement.odometer.enabled and odometerMeasurement.odometer is not None:
-                        self.trip.end_mileage_km = odometerMeasurement.odometer.value
+            if (flags & AddressableLeaf.ObserverEvent.ENABLED):
+                if self.trip is None:
+                    time = datetime.utcnow().replace(tzinfo=timezone.utc, microsecond=0) - timedelta(seconds=self.updateInterval)
+                    self.trip = Trip(self.vehicle, time, None, None, None, None)
+                    if self.vehicle.weConnectVehicle.statusExists('measurements', 'odometerStatus') \
+                            and self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus'].enabled:
+                        odometerMeasurement = self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus']
+                        if odometerMeasurement.odometer.enabled and odometerMeasurement.odometer is not None:
+                            self.trip.start_mileage_km = odometerMeasurement.odometer.value
+                    with self.session.begin_nested():
+                        try:
+                            self.session.add(self.trip)
+                        except IntegrityError as err:
+                            LOG.warning('Could not add trip to the database, this is usually due to an error in the WeConnect API (%s)', err)
+                    self.session.commit()
+                    LOG.info(f'Vehicle {self.vehicle.vin} started a trip')
+            elif (flags & AddressableLeaf.ObserverEvent.DISABLED):
+                if self.trip is not None:
+                    with self.session.begin_nested():
+                        self.trip.endDate = datetime.utcnow().replace(tzinfo=timezone.utc, microsecond=0)
+                        if self.vehicle.weConnectVehicle.statusExists('measurements', 'odometerStatus') \
+                                and self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus'].enabled:
+                            odometerMeasurement = self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus']
+                            if odometerMeasurement.odometer.enabled and odometerMeasurement.odometer is not None:
+                                self.trip.end_mileage_km = odometerMeasurement.odometer.value
+                    self.session.commit()
 
-                self.session.commit()
-                self.trip = None
+                    self.trip = None
 
-                LOG.info(f'Vehicle {self.vehicle.vin} ended a trip')
-            else:
-                if flags is not None:
-                    LOG.info(f'Vehicle {self.vehicle.vin} reports to be inactive, but no trip was started (this is ok during startup)')
+                    LOG.info(f'Vehicle {self.vehicle.vin} ended a trip')
+                else:
+                    if flags is not None:
+                        LOG.info(f'Vehicle {self.vehicle.vin} reports to be inactive, but no trip was started (this is ok during startup)')
 
     def __onPlugConnectionStateChanged(self, element, flags):  # noqa: C901
         if self.trip is not None:
@@ -295,21 +320,24 @@ class TripAgent():
             except ObjectDeletedError:
                 LOG.warning('Last trip entry was deleted')
                 self.trip = None
+            except InvalidRequestError:
+                LOG.warning('Last trip entry was not persisted')
 
         plugStatus = self.vehicle.weConnectVehicle.domains['charging']['plugStatus']
         if element.value == PlugStatus.PlugConnectionState.CONNECTED:
             if self.trip is not None:
-                if plugStatus.carCapturedTimestamp.enabled:
-                    self.trip.endDate = plugStatus.carCapturedTimestamp.value
-                else:
-                    self.trip.endDate = datetime.utcnow().replace(tzinfo=timezone.utc, microsecond=0)
-                if self.vehicle.weConnectVehicle.statusExists('measurements', 'odometerStatus') \
-                        and self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus'].enabled:
-                    odometerMeasurement = self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus']
-                    if odometerMeasurement.odometer.enabled and odometerMeasurement.odometer is not None:
-                        self.trip.end_mileage_km = odometerMeasurement.odometer.value
-
+                with self.session.begin_nested():
+                    if plugStatus.carCapturedTimestamp.enabled:
+                        self.trip.endDate = plugStatus.carCapturedTimestamp.value
+                    else:
+                        self.trip.endDate = datetime.utcnow().replace(tzinfo=timezone.utc, microsecond=0)
+                    if self.vehicle.weConnectVehicle.statusExists('measurements', 'odometerStatus') \
+                            and self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus'].enabled:
+                        odometerMeasurement = self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus']
+                        if odometerMeasurement.odometer.enabled and odometerMeasurement.odometer is not None:
+                            self.trip.end_mileage_km = odometerMeasurement.odometer.value
                 self.session.commit()
+
                 self.trip = None
 
                 LOG.info(f'Vehicle {self.vehicle.vin} ended a trip (car was connected to charger)')
@@ -329,13 +357,13 @@ class TripAgent():
                             odometerMeasurement = self.vehicle.weConnectVehicle.domains['measurements']['odometerStatus']
                             if odometerMeasurement.odometer.enabled and odometerMeasurement.odometer is not None:
                                 self.trip.start_mileage_km = odometerMeasurement.odometer.value
-                        try:
-                            with self.session.begin_nested():
+                        with self.session.begin_nested():
+                            try:
                                 self.session.add(self.trip)
-                            self.session.commit()
-                        except IntegrityError as err:
-                            LOG.warning('Could not add trip to the database, this is usually due to an error in the WeConnect API (%s)', err)
+                            except IntegrityError as err:
+                                LOG.warning('Could not add trip to the database, this is usually due to an error in the WeConnect API (%s)', err)
+                        self.session.commit()
                         LOG.info(f'Vehicle {self.vehicle.vin} started a trip (car was disconnected from charger)')
 
     def commit(self):
-        pass
+        self.session.commit()
